;; Generated package description from avy.el
(define-package "avy" "0.4.0" "tree-based completion" '((emacs "24.1") (cl-lib "0.5")) :url "https://github.com/abo-abo/avy" :keywords '("point" "location"))
